% *************************************************************************
% *** Latent Manifold ProMPs Demo ***
%
% Loads real robot trajectories 
% Learns a distribution from it
% And computes predictions given some observations
%
% Paper: Rueckert, E.; Mundo, J.; Paraschos, A.; Peters, J.; Neumann, G (2015). 
%        Extracting Low-Dimensional Control Variables for Movement Primitives, 
%        Proceedings of the International Conference on Robotics and
%        Automation (ICRA).
%
% Last Update: April, 2015
% Author: Rueckert Elmar, rueckert@ias.tu-darmstadt.de 
% *************************************************************************
clear all;
close all;
clc;
addpath([pwd filesep 'LMProMPs']);

%FLAGS
flgPlotDataSet = 0;

%Parameters for the Bi modal reaching task
hyperparameters.flgEstHyperFromData = 0;   %re-estimate the hyperparams from the data
hyperparameters.flgUseKMeans = 0;   %override the initial cluster assignment by kmeans
hyperparameters.maxDimLatent = 1;

hyperparameters.a0 = 1;
hyperparameters.b0 = 1;
hyperparameters.gamma = 1;
hyperparameters.c0 = 1; 
hyperparameters.d0 = 0.3;
    

if ~hyperparameters.flgEstHyperFromData 
    hyperparameters.lambda0 = 1;
    hyperparameters.beta = 50;

    hyperparameters.lambdal = hyperparameters.c0 / hyperparameters.d0;
end
%Variational inference parameters
algparameters.EMSteps = 100;
algparameters.flgPrintMsgs = 1; %print incremental results
algparameters.learnClusterAssignments = 0;
    
    
fileName = [pwd filesep 'datasets' filesep 'RobotBiModal' filesep '7DKukaCartReachingTask.mat'];
load(fileName);
NumGaussians = n;


%% PLOT the dataset
if flgPlotDataSet
    curPath = pwd;
    cd([pwd filesep 'datasets']);
    plot_datasetsSubFig( fileName );
    cd(curPath);
end

%% Learn the initial model through Least Squares Regression
[model, w, SN, zN] = LMProMPs_init(datasets, hyperparameters, DOF, NumGaussians);
LMProMPs_plot_prior(model, zN, datasets, 2); %Plot the model

%% Learn the model through Variational Inference
[viModel, wN, SN, zN, K, trainError] = LMProMPs_train (datasets, model, w, SN, zN, algparameters);
figure;
plot(trainError);

LMProMPs_plot_prior(viModel, zN, datasets, 2); %Plot the model

