function LMProMPs_plot_prior(model, zN, datasets, dimensionToPlot)
%This function plots the prior distribution, the effect of the latent
%variables and the true demonstrations if the second argument (datasets) is
%provided.

    if nargin < 3
        datasets = [];
    end
    if nargin < 4
        dimensionToPlot = 1:model.DoF;
    end
    
    %for each degree of freedom
    for d=dimensionToPlot
        LMProMPs_plot_priorPerDoF(d, model, datasets);
    end 

    %plot the latent variables
    hFigLatent = figure;
    set(hFigLatent,'Color','white');
    clusterCols = lines(model.K);
    hold all;
%     keyboard
    if model.maxDimLatent == 1
        %1D Bar Plots
        for k=1:model.K
            hs = sort(model.comp(k).h(find(zN(:,k)>0)));
            plot(hs, 'color', clusterCols(k,:), 'linewidth', 3);
        end
        legend('components');
        xlabel('demonstratios');
        ylabel('latent variable');
    elseif model.maxDimLatent == 2
        %keyboard
        for k=1:model.K
            hs = model.comp(k).h(:,find(zN(:,k)>0));
            plot(hs(1,:), hs(2,:), '.', 'color', clusterCols(k,:), 'linewidth', 3);
        end
        legend('components');
        xlabel('h_1');
        ylabel('h_2');
            
        %take the two most significant ones in a 2d Plot
%         error('not implemented yet')
    end
    
    
end



function LMProMPs_plot_priorPerDoF(dimensionToPlot, model, datasets)

%Formatting parameters
fs = 26;
fsLegend = 18;
fsLegendKl=12;
lw = 3;
lwSmall = 1.5;
markers = {'o','s','d','^','pentagram','hexagram','+','x'};

%% trajectory plots using wl

        
for k=1:model.K
    hFigFeaturesTraj = figure;
    set(hFigFeaturesTraj,'Color','white');
    hold all;

    sigma_star = model.comp(k).alpha^(-1)*eye(model.M) + model.comp(k).wl*(model.comp(k).gammaN^-1)*model.comp(k).wl';
    hMinMax =  0.8*(max(model.comp(k).h')'-min(model.comp(k).h')')/2; %80 percent of the extremes
    coordIds = dimensionToPlot:model.DoF:size(datasets(1).Y,1);
    xAxisIds = linspace(0,1,length(coordIds));    
    
    X_test = datasets(1).PSI_X;
    %plot the prior
    for dimLatentId = 1:model.maxDimLatent
        w = model.comp(k).w0 + hMinMax(dimLatentId)*model.comp(k).wl(:,dimLatentId);
        y_mean = X_test * w;

        y_predVar = zeros(size(y_mean(coordIds)));
        for t=1:length(coordIds)
            y_predVar(t,:) =  datasets(1).PSI_X(coordIds(t),:)*sigma_star*datasets(1).PSI_X(coordIds(t),:)' + model.beta^(-1);
        end
        lndPred =  shadedErrorBar(xAxisIds,y_mean(coordIds),sqrt(y_predVar),'k',1);
        est_plot = lndPred.mainLine;
        set(est_plot, 'linewidth', lwSmall);
    end  

    %plot the data
    if ~isempty(datasets)
        clusterCols = lines(datasets(1).NumCluster);
        
        for i=1:model.L
            coordIds = dimensionToPlot:model.DoF:size(datasets(i).Y,1);
            xAxisIds = linspace(0,1,length(coordIds)); 
            
            plot(xAxisIds, datasets(i).Y(coordIds)','color', clusterCols(datasets(i).clusterId,:), 'linewidth', 1);
        end
    end
    hold off;
    v=axis;
    text(100,v(4),sprintf('comp. %d',k),'fontsize',fs);
    set(gca, 'fontsize', fs);
    xlabel('time step', 'fontsize', fs);
    ylabel(['dim. ' num2str(dimensionToPlot)], 'fontsize', fs);
%         %axis tight

end


end