function [mk, wN, SN, zN, K, trainError] = LMProMPs_train (datasets, model, wN, SN, zN, algparameters)


    %datasets, EMSteps, V, K, plott,...
    %dp_alpha, debug, compute_lb, use_approx_hypers,old_vi_file, a0,b0,lambda0,gamma,beta,c0,d0,betaIn)
    EMSteps = algparameters.EMSteps;
    debug = algparameters.flgPrintMsgs;

    K = model.K;     %(initial) number of clusters
    L = model.L;           %number of demonstrations in total
    M = model.M;  % must be equal for all tasks: DOF * NumberOfGaussians
    V = model.maxDimLatent;
    
    EM_diff = 10^-10;
    lb_last = -Inf;
    
    % precompute training_data matrices
    X_Y = zeros(M,L);
    X_X = zeros(M,M,L);
    
    for i=1:L
        x_tmp = datasets(i).PSI_X;
        X_Y(:,i) = sum(x_tmp' * datasets(i).Y,2); % if not all samples in task have the same basisfunctions -> change here
        X_X(:,:,i) = x_tmp' * x_tmp;
    end    
    
    % robustness for only little task considered
    K = min(K,L);
    
    % for debugging
    if algparameters.learnClusterAssignments 
        updateZ =   1;
        updatePi =  1;
    else
        updateZ =   0;
        updatePi =  0;
    end
    updateW =   1;
    updateW0L = 1;
    updateA =   1;
    updateH =   1;
    
       
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% variational inference
    %%%%%%%%%%%%%%%%%%%%%%%%%%%

    mk = model;
    %vi = model;
        
    lb = zeros(1,EMSteps);
    trainError = zeros(EMSteps,L);
  
    % iteratively update 
    for t=1:EMSteps 
        if debug
            display(sprintf('learn itearation: %d/%d',t,EMSteps));
        end
        
%         % add a new mixture component depending on the actual
%         % responsibilities
%         if dp_alpha > 0
%             dp_threshold = 1;
%             % compute responsibilities for each component and datapoint
%             if t > 5 && t  < EMSteps - 20
%                 resp = zeros(L,K);
%                 resp_us = zeros(L,K);
%                 smalls= [];
%                 for i=1:L
%                     for k = 1:K
%                        resp(i,k) = mk.comp(k).pi * mvnpdf(wN(:,i),mk.comp(k).w0 + mk.comp(k).wl * mk.comp(k).h(:,i),mk.comp(k).alpha^-1 * eye(M)) * M;
%                        resp_us(i,k) = mk.comp(k).pi * mvnpdf(wN(:,i),mk.comp(k).w0 ,mk.comp(k).alpha^-1 * eye(M)) * M;
%                     end
%                 end
% 
%                 resp_norm = zeros(L,K);
%                 for i=1:L
%                     % numerical problems with normalizing
%                    for k=1:K
%                        resp_norm(i,k) = resp(i,k) / sum(resp(i,:),2);
%                    end
%                    if sum(resp(i,:),2) < dp_threshold
%                       smalls = [smalls, i]; 
%                    end
%                 end
% 
%                 if length(smalls) > 0 && rand < dp_alpha
%                    [mk,zN] = MDPM_extend_data( mk,wN,zN,smalls); 
%                    K = K+1;
%                 end
%             end
%         end
        
        % remove components with 0 responsibility
        if 1==0
            components_to_remove = find([mk.comp.pi] == 0);
            mk.comp(components_to_remove) = [];
            zN(:,components_to_remove) = [];
            K = K - length(components_to_remove);
        end
        
                
        %1. first estimate the expectation of z
        if updateZ
            tmp_z = zeros(K,L);
            r = zeros(K,L);   
            for k=1:K
                c = mk.comp(k);
                for i=1:L
                    tmp_z(k,i) =(wN(:,i) - c.w0 - c.wl * c.h(:,i))' * (wN(:,i) - c.w0 - c.wl * c.h(:,i)) + trace(SN(:,:,i)) ...
                        + M * c.lambda0N^-1 + c.h(:,i)' * c.LV * c.h(:,i) + trace((c.wl' * c.wl + M * c.LV) * c.gammaNInv);
                    
%                     r(k,i) = log(c.pi) + 0.5 * M * log(c.alpha) - 0.5 * c.alpha * tmp_z(k,i);
                    r(k,i) = log(c.pi) + 0.5 * M * (psi(c.aN) - log(c.bN)) - 0.5 * c.alpha * tmp_z(k,i);
                end
            end

            % for robustness
            if 1 == 1
                r_robust = r;
                for i=1:L
                   r_robust(:,i) = exp(r(:,i) - max(r(:,i))); 
                end
                rz = r_robust;
            else 
                rz = exp(r);
            end
            for k=1:K
                zN(:,k) = rz(k,:) ./ sum(rz,1);
%                 zN(isnan(zN(:,k)),k) = 0; % bad initialization
            end
         end
         
                 

        %2. compute model vectors
%         keyboard
        if updateW
           for i=1:L
                SN(:,:,i) = mk.beta * X_X(:,:,i) + sum(zN(i,:) .* [mk.comp.alpha])* eye(M);
                
                wN(:,i) = SN(:,:,i) \ (mk.beta*X_Y(:,i) + ...
                    sum(bsxfun(@times, zN(i,:).*[mk.comp.alpha],cell2mat(arrayfun(@(c) (c.w0 + c.wl * c.h(:,i)),mk.comp,'un',0))),2));
                SN(:,:,i) = inv(SN(:,:,i));
                
                if 1==1 %compute the training error
                    x_tmp = datasets(i).PSI_X;
                    yPred = x_tmp * wN(:,i);
                    yTrue = datasets(i).Y;
                    trainError(t,i) = sqrt(sum((yTrue - yPred).^2))/length(yPred);
                end
           end           
        end
        
        %3. model vector prior mean
        if updateW0L
            for k = 1:K
                c = mk.comp(k);

                % w0
                mk.comp(k).lambda0N = c.alpha * sum(zN(:,k)) + mk.lambda0;     
                mk.comp(k).w0 = mk.comp(k).lambda0N^-1 * c.alpha * sum(bsxfun(@times,zN(:,k)', (wN - c.wl * c.h)),2);
                
                %lambdal
                mk.comp(k).cN = mk.c0 + 0.5 * M;
                for v=1:V
                    mk.comp(k).dN = mk.d0 + 0.5 * mk.comp(k).wl(:,v)' * mk.comp(k).wl(:,v);
                    mk.comp(k).lambdalN(v) = mk.comp(k).cN / mk.comp(k).dN;
                end


                %wl
                for v=1:V
                    % TODO optimize performance
                    tmp = 0;
                    for i=1:L
                       tmp = tmp + zN(i,k) *(c.h(v,i)' * c.h(v,i) + c.gammaNInv(v,v));
                    end
                    mk.comp(k).LV(v,v) = 1 / (mk.comp(k).lambdalN(v) + c.alpha * tmp);
                    
                    tmp = 0;
                    for i=1:L
                       tmp = tmp + (zN(i,k) * eye(M)) * (wN(:,i) - mk.comp(k).w0) * c.h(v,i)';
                    end
                    mk.comp(k).wl(:,v) = mk.comp(k).LV(v,v) * c.alpha * tmp;
                end
            end  
        end
        
        %4. latent variable h
        if updateH
            for k = 1:K
                c = mk.comp(k);
                mk.comp(k).gammaN = c.alpha * (c.wl' * c.wl + M * c.LV) + mk.gamma * eye(V);
                            
                %compute the inverse of gamma using SVD and Levenberg–Marquardt regurlarization
                [Usvd,Ssvd,Vsvd] = svd(mk.comp(k).gammaN);
                SsvdInv = diag( diag(Ssvd).^-1 );
                mk.comp(k).gammaNInv = Usvd*SsvdInv*Vsvd'; % eigentliches gammaN
                
%                 mk.comp(k).gammaNInv = mk.comp(k).gammaN;
            
                for i=1:L
                    mk.comp(k).h(:,i) = mk.comp(k).gammaNInv * (c.alpha * zN(i,k) * c.wl' * (wN(:,i) - c.w0));
                end
                %mk.comp(k).h
            end
        end
        
        %5. model vector prior covariance
        if updateA
            tmp_z = zeros(K,L);
            for k=1:K
                c = mk.comp(k);
                for i=1:L
                    tmp_z(k,i) = (wN(:,i) - c.w0 - c.wl * c.h(:,i))' * (wN(:,i) - c.w0 - c.wl * c.h(:,i)) + trace(SN(:,:,i)) ...
                        + M * c.lambda0N^-1 + c.h(:,i)' * M * c.LV * c.h(:,i) + trace((c.wl' * c.wl + M * c.LV) * c.gammaNInv);
                end
            end
            for k = 1:K           
                mk.comp(k).aN = mk.a0 + 0.5 * M * sum(zN(:,k));
                mk.comp(k).bN = mk.b0 + 0.5 * sum(zN(:,k)' .* tmp_z(k,:));
                mk.comp(k).alpha = mk.comp(k).aN/mk.comp(k).bN;
            end
        end           
        
        % M-Step pi
        if updatePi
            for k = 1:K
                mk.comp(k).pi = 1/L*(sum(zN(:,k)));
            end
        end

    end
end

