function [ w_star, sigma_star, h_star, gamma_star, z_star] = MDPM_conditioning( X_star,y_star,vi, EMSteps, plott, lambda, DDB)
% do the prediction for the mixture-model by conditioning

    K = size(vi.mk.comp,2); 
    M = size(vi.w,1);
    V = size(vi.mk.comp(1).h,1);
    S = size(X_star,1) / DDB;
    lb = zeros(1,EMSteps);
    
    resp = zeros(1,K);
    z_star = zeros(1,K);

    % initializing
    if size(lambda,1)*size(lambda,2) == 1
        lambdaMat = eye(DDB)* lambda;
    else
        lambdaMat = diag(lambda);
    end
    
    
    
    for k=1:K
        if 1==0
            vi.mk.comp(k).h_star = median(vi.mk.comp(k).h,2);
        else
            hMinMax =  (max(vi.mk.comp(k).h')'-min(vi.mk.comp(k).h')')/2;
            vi.mk.comp(k).h_star = hMinMax;
        end
        
        if 1==0
            [Usvd,Ssvd,Vsvd] = svd(vi.mk.comp(k).gammaN);
            SsvdInv = diag( diag(Ssvd).^-1 );
            vi.mk.comp(k).h_var = Usvd*SsvdInv*Vsvd';
        else
            vi.mk.comp(k).h_var = inv(vi.mk.comp(k).gammaN);
        end

        vi.mk.comp(k).sigma_star = vi.mk.comp(k).alpha^(-1)*eye(M);% + vi.mk.comp(k).wl*vi.mk.comp(k).h_var*vi.mk.comp(k).wl';
        %vi.mk.comp(k).w_star = vi.mk.comp(k).w0 + vi.mk.comp(k).wl*vi.mk.comp(k).h_star;
    end

    % ------------------------------------------------------------
    for t=1:EMSteps     
        
        for k=1:K
            %update h - conditioning
            for s = 1:S
                x_tmp = X_star((s-1)*DDB+1:s*DDB,:)';
                y_tmp = y_star((s-1)*DDB+1:s*DDB,:);  
                features = x_tmp';

                tempMatrix = vi.mk.comp(k).h_var'*vi.mk.comp(k).wl'*features' / (lambdaMat + features * vi.mk.comp(k).sigma_star * features' + ...
                    features*vi.mk.comp(k).wl*vi.mk.comp(k).h_var'*vi.mk.comp(k).wl'*features');
                newH = vi.mk.comp(k).h_star + tempMatrix * (y_tmp - features * (vi.mk.comp(k).w0 + vi.mk.comp(k).wl * vi.mk.comp(k).h_star));

                newHVar = vi.mk.comp(k).h_var - tempMatrix * features * vi.mk.comp(k).wl * vi.mk.comp(k).h_var;

                vi.mk.comp(k).h_star = newH;
                vi.mk.comp(k).h_var = newHVar;
            end
            
            % update w - conditioning
            vi.mk.comp(k).sigma_star = vi.mk.comp(k).alpha^(-1)*eye(M) + vi.mk.comp(k).wl*vi.mk.comp(k).h_var*vi.mk.comp(k).wl';
            vi.mk.comp(k).w_star = ( vi.mk.comp(k).w0 + vi.mk.comp(k).wl*vi.mk.comp(k).h_star );  
            
            lb(t) = lb(t) + sum(newH);
        end
        
    end
        
%         keyboard
    % update responsibilities
    for k=1:K
        %     if not symmetric only take diagonal
        if  sum(sum(vi.mk.comp(k).sigma_star ~= vi.mk.comp(k).sigma_star')) ~= 0
            %display ('problem: sigma_star is not symmetric')
            resp(k) = vi.mk.comp(k).pi * mvnpdf(vi.mk.comp(k).w_star, vi.mk.comp(k).w0 , ...
                diag(diag(vi.mk.comp(k).sigma_star))) * M;
        else
            resp(k) = vi.mk.comp(k).pi * mvnpdf(vi.mk.comp(k).w_star, vi.mk.comp(k).w0 , ...
               vi.mk.comp(k).sigma_star) * M;
        end
    end
    % if no component is responsible shift all priors with h_star and try
    % again
    if sum(resp) == 0
        for k=1:K
            if  sum(sum(vi.mk.comp(k).sigma_star ~= vi.mk.comp(k).sigma_star')) ~= 0
                display ('problem: sigma_star is not symmetric')
                resp(k) = vi.mk.comp(k).pi * mvnpdf(vi.mk.comp(k).w_star, vi.mk.comp(k).w0 + vi.mk.comp(k).wl * vi.mk.comp(k).h_star, ...
                    diag(diag(vi.mk.comp(k).sigma_star))) * M;
            else
                resp(k) = vi.mk.comp(k).pi * mvnpdf(vi.mk.comp(k).w_star, vi.mk.comp(k).w0 + vi.mk.comp(k).wl * vi.mk.comp(k).h_star, ...
                    vi.mk.comp(k).sigma_star) * M;
            end
        end
    end
    % update z - normalize responsibilities
    for k=1:K
        z_star(k) = resp(k) / sum(resp);
    end
%     z_star = [1 0];
    
    if 1 == 0
        % determine active component: only single component with maximum impact
        [~, active_comp] = max(z_star);
        w_star = vi.mk.comp(active_comp).w_star;
        sigma_star = vi.mk.comp(active_comp).sigma_star;
        h_star = vi.mk.comp(active_comp).h_star;
        [Usvd,Ssvd,Vsvd] = svd(vi.mk.comp(active_comp).h_var); 
        SsvdInv = diag( diag(Ssvd).^-1 );
        gamma_star = Usvd*SsvdInv*Vsvd';
    else
        % determine active component - components weighted by responsibility
        w_star = [vi.mk.comp.w_star] * z_star';
        h_star = [vi.mk.comp.h_star] * z_star';
        
        sigma_star = zeros(M,M); 
        gamma_star = zeros(V,V);
        for k = 1:K
            sigma_star = sigma_star + vi.mk.comp(k).sigma_star .* z_star(k);
            [Usvd,Ssvd,Vsvd] = svd(vi.mk.comp(k).h_var); 
            SsvdInv = diag( diag(Ssvd).^-1 );
            gamma_star = gamma_star + Usvd*SsvdInv*Vsvd' .* z_star(k);
        end
    end
        
    if plott
       figure
       hold all
       xlabel('# EMSteps');
       ylabel('Lower bound');
       plot(lb);
    end
end

