function lb = MDPM_lower_bound( w,SN,zN,mk,datasets,debug)
    % computes the lower bound, but leaves out constant terms because they
    % are not needed
    
    M = size(w,1);
    L = size(w,2);
    K = size(zN,2);
    V = size(mk.comp(1).h,1);
    
    lb = 0;
    
    % p(D|w,beta)
    for i=1:L
        S = size(datasets(i).Y,1);
        lb = lb - (S/2) * log(mk.beta) - mk.beta * ((datasets(i).Y - datasets(i).PSI_X * w(:,i))' * ...
            (datasets(i).Y - datasets(i).PSI_X * w(:,i)) + trace(datasets(i).PSI_X' * datasets(i).PSI_X * squeeze(SN(:,:,i))));
    end
    if debug
        display(sprintf('p(D|w,beta): lb=%d',lb));
    end
    
    % p(w|wo,wl,h,alpha)
    for i=1:L
        for k=1:K
            lb = lb + zN(i,k) * ((-M/2) * (psi(mk.comp(k).aN) - log(mk.comp(k).bN)) - (mk.comp(k).alpha / 2) ...
                * ((w(:,i) - mk.comp(k).w0 - mk.comp(k).wl * mk.comp(k).h(:,i))' * (w(:,i) - mk.comp(k).w0 - mk.comp(k).wl * mk.comp(k).h(:,i)) ...
                + trace(squeeze(SN(:,:,i))) + M / mk.comp(k).lambda0N + trace(mk.comp(k).wl' * mk.comp(k).wl * inv(mk.comp(k).gammaNMat(i).task)) ...
                + (M*V / mk.comp(k).lambdalN)*(mk.comp(k).h(:,i)' * mk.comp(k).h(:,i) + trace(inv(mk.comp(k).gammaNMat(i).task))) ));
        end
    end
    if debug
        display(sprintf('p(w|wo,wl,h,alpha): lb=%d',lb));
    end
    
    % p(h|gamma)
    for i=1:L
        for k=1:K
            lb = lb - (V/2) * log(mk.gamma) - (mk.gamma/2)* (mk.comp(k).h(:,i)' * mk.comp(k).h(:,i) + trace(mk.comp(k).gammaNMat(i).task));
        end
    end
    if debug
        display(sprintf('p(h|gamma): lb=%d',lb));
    end
    
    % p(w0|lambda0)
    for k=1:K
       lb = lb - (M/2) * log(mk.lambda0) - (mk.lambda0 /2) * (mk.comp(k).w0' * mk.comp(k).w0 + M * mk.comp(k).lambda0N);
    end
    if debug
        display(sprintf('p(w0|lambda0): lb=%d',lb));
    end

    % p(Wl|lambdal)
    for k=1:K
        wl = mk.comp(k).wl(:);
        lb = lb - (M*V/2) * log(mk.lambdal) - (mk.lambdal /2) * (wl' * wl + M*V * mk.comp(k).lambdalN);
    end
    if debug
        display(sprintf('p(Wl|lambdal): lb=%d',lb));
    end
    
    % p(alpha|a0,b0)
    for k=1:K
       lb = lb - gammaln(mk.a0) + mk.a0 * (log(mk.b0/mk.comp(k).bN) + psi(mk.comp(k).aN)) - ...
           psi(mk.a0) + log(mk.comp(k).bN) - mk.b0 * (mk.comp(k).alpha);
    end
    if debug
        display(sprintf('p(alpha|a0,b0): lb=%d',lb));
    end
    
     % p(Z|pi)
    for i=1:L
        for k=1:K
            if zN(i,k) ~= 0
                lb = lb + zN(i,k) * log(mk.comp(k).pi);
            end
        end
    end
    if debug
        display(sprintf('p(Z|pi): lb=%d',lb));
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    
    % q(w): log(det(S)) = sum(log(eig(S)));
    for i=1:L
        eigenvalues = eig(squeeze(SN(:,:,i)));
        lb = lb - ( (-M/2) * sum(log(eigenvalues))) ;
    end
    if debug
        display(sprintf('q(w): lb=%d',lb));
    end
    

        
    % q(w0)
    for k=1:K
        lb = lb - ( (-M/2) * log(mk.comp(k).lambda0N^-1));
    end
    if debug
        display(sprintf('q(w0): lb=%d',lb));
    end
    
    % q(Wl)
    for k=1:K
       lb = lb - ( (-M * V / 2) * log(mk.comp(k).lambdalN^-1)); 
    end
    if debug
        display(sprintf('q(Wl): lb=%d',lb));
    end
          
    % q(h)
    for i=1:L
        for k=1:K
            lb = lb - ( (-V/2) * sum(log(eig(inv(mk.comp(k).gammaNMat(i).task)))));
        end
    end  
    if debug
        display(sprintf('q(h): lb=%d',lb));
    end
    
    % q(alpha)
    for k=1:K
       lb = lb - (log(mk.comp(k).bN) - gammaln(mk.comp(k).aN) + (mk.comp(k).aN - 1) * psi(mk.comp(k).alpha) - mk.comp(k).aN); 
    end
    if debug
        display(sprintf('q(alpha): lb=%d',lb));
    end
    
    % q(Z)
    for i=1:L
        for k=1:K
            if zN(i,k) ~= 0
                lb = lb - zN(i,k) * log(zN(i,k));
            end
        end
    end
    if debug
        display(sprintf('q(Z): lb=%d',lb));
    end  
end

